//
//   @file    fastnlo_utils.cxx         
//   
//
//   @author M.Sutton
// 
//   Copyright (C) 2016 M.Sutton (sutt@cern.ch)    
//
//   $Id: fastnlo_utils.cxx, v0.0   Mon 25 Apr 2016 15:03:20 CEST sutt $

#include "amconfig.h"

#ifdef HAVE_FASTNLO

#include <iostream>


#include "nnlo_common.h"
#include "nnlo_utils.h"

#include "fastnlo_utils.h"

/// fastnlo headers ...
#include "fastnlotk/fastNLOCreate.h"
#include "fastnlotk/read_steer.h"



static fastNLOCreate *ftable;

// Temporary pointer to APPLGRID lumi function for fastNLO
const lumi_pdf *fnlopdf;


void nnlo::init_fastnlo( const int& id, std::vector<double>& binning ) {

  // Book fastNLO table
  
  //      using namespace std;
  std::cout << "### nnlo::init(): Book fastNLO table ..." << std::endl;

  int leading_order = inppar_.njets;
  
  // Set parton lumi mappings
  // For the time being use Mark's APPLGRID trunk update to map colour dependent channels to subprocesses
  // LO for testing; use the one created in APPLGRID booking if it exists
  fnlopdf = dynamic_cast<const lumi_pdf*>( appl::appl_pdf::getpdf("ZJ-LO.config") );
  // Create new mapping
  if ( fnlopdf == 0 ) {
    lumi_pdf* _fnlopdf = new lumi_pdf("ZJ-LO.config");
    _fnlopdf->removeDuplicates();
    fnlopdf = _fnlopdf;
  };
  std::vector<std::vector<int> > PartonCombinationsLO = fnlopdf->vectorise();
  
  
  const std::string steeringNameSpace = "fastNLObase.str";
  
  int nprocs = PartonCombinationsLO.size();
  
  
  // Values passed here will overwrite a possible value in the steering
  std::cout << "fastNLO initialisation"        << std::endl;
  std::cout << "Steering name space set to: "  << steeringNameSpace << std::endl;
  std::cout << "FASTNLO: Book fastNLO interpolation grid." << std::endl;
  std::cout << "FASTNLO: Set parameters that are passed from NNLOJETS to fastNLO!" << std::endl;
  
  // Pass process info
  // LO & copy for now
  ADD_NS("LeadingOrder", leading_order, steeringNameSpace);
  
  ADD_NS("IPDFdef3LO",nprocs,steeringNameSpace);
  ADD_NS("IPDFdef3NLO",nprocs,steeringNameSpace);
  ADD_NS("IPDFdef3NNLO",nprocs,steeringNameSpace);
  ADD_NS("NSubProcessesLO",nprocs,steeringNameSpace);
  ADD_NS("NSubProcessesNLO",nprocs,steeringNameSpace);
  ADD_NS("NSubProcessesNNLO",nprocs,steeringNameSpace);

  std::vector<std::string> Tmp;
  ADDTABLE_NS("PartonCombinationsLO",Tmp,PartonCombinationsLO,steeringNameSpace);
  ADDTABLE_NS("PartonCombinationsNLO",Tmp,PartonCombinationsLO,steeringNameSpace);
  ADDTABLE_NS("PartonCombinationsNNLO",Tmp,PartonCombinationsLO,steeringNameSpace);
  
  // Pass binning
  ADDARRAY_NS("SingleDifferentialBinning", binning, steeringNameSpace);
  
  // Create table
  ftable = new fastNLOCreate("fastNLObase.str",
			     steeringNameSpace,
			     true);

  std::cout << "FASTNLO: fastNLO grid booked." << std::endl;
  
}




void nnlo::fill_fastnlo( const int& id, 
			 const double& x1, const double& x2, const double& Q2, 
			 const double& obs, 
			 const double* wt ) { 

  // Fill fastNLO table

  //      std::cout << "### nnlo::fill(): Fill fastNLO table ..." << std::endl;
  
  int leading_order = inppar_.njets;
  int iorder   = ndimcurrent_.norder;
  int nv       = ndimcurrent_.nv;
  int iproc    = currentprocess_.iproc;
  int iprocess = fnlopdf->decideSubProcess( iproc );
  //      std::cout << "iorder = " << iorder << ", nv = " << nv << ", iproc = " << iproc << ", iprocess = " << iprocess << std::endl;
  ftable->SetOrderOfAlphasOfCalculation(leading_order+iorder);
  ftable->fEvent.SetProcessId(iprocess);
  ftable->fEvent.SetWeight(wt[0]);
  ftable->fEvent.SetX1(x1);
  ftable->fEvent.SetX2(x2);
  ftable->fScenario.SetObservable0(obs);
  ftable->fScenario.SetObsScale1(sqrt(Q2));
  ftable->Fill(0);
  ftable->fEvent.Reset();
}


void fill_ref_fastnlo( const int& id, const double& obs, const double* wt ) {

}




void nnlo::term_fastnlo( const int& id, int nweights ) { 

  std::cout << "### nnlo::term(): Write fastNLO table ..." << std::endl;
  unsigned long long int nEvents = nweights;
  ftable->SetNumberOfEvents(nEvents);
  if (ftable->GetIsWarmup()) {
    std::cout << "### nnlo::term(): fastNLO warmup run finished ..." << std::endl;
    ftable->WriteTable();
  } else {
    std::cout << "### nnlo::term(): fastNLO run finished ..." << std::endl;
    ftable->WriteTable();
  }
  
}



#endif
